#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <glib.h>

#include "Cora.h"
#include "cpgplot.h"

#define MOUSE_R 'X'
#define MOUSE_M 'D'
#define MOUSE_L 'A'

#define ERROR(c) { g_warning ("%s\n", (c)); \
return (-1);} 

#ifndef MAX
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#endif
#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif

/* -- for smoothing/rebinning --
 */
#define S_AVERAGE 0
#define S_MEDIAN  1
#define S_REBIN   2
static int smooth_mode = S_AVERAGE;
static int smooth      = 1;
static float zoom      = 1.0;


void redraw_it (float * xposition, float yposition, int lines, 
                float * wave, float * flux, int count,
                float uxmin, float uxmax, float uymin, float uymax)
{
  int i;
  double dx,dy;
  char smfac[64];
  
  cpgend();
  if(cpgopen("/XSERVE") != 1)
    {
      g_warning("Cannot open plot device");
      return;
    }
  
  
  cpgenv(uxmin, uxmax, uymin, uymax, 0, 1);
  cpglab("\\gl [\\A]", "Counts/bin", "");
  
  cpgbin(count, wave, flux, 1);
  cpgpt (count, wave, flux, 1);
  
  cpgsci(3);
  for (i = 0; i < lines; ++i)
    {
      cpgmove(xposition[i], uymin);
      cpgdraw(xposition[i], uymax);
    }
  cpgsci(1);
  if (yposition != 0.)
    {
      cpgsci(2);
      cpgmove(uxmin, yposition);
      cpgdraw(uxmax, yposition);
      cpgsci(1);
    }
  dx = uxmax - uxmin;
  dy = uymax - uymin;

  cpgtext(uxmin, uymax+.10*dy, "Left  Mouse Button: Select range");
  cpgtext(uxmin, uymax+.07*dy, "Right Mouse Button: Select line");
  cpgtext(uxmin, uymax+.04*dy, "d:    Undo selection of last line");
  cpgtext(uxmin, uymax+.01*dy, "b:    Select Background");
  
  cpgtext(uxmin+.65*dx, uymax+.10*dy, "q:     quit        c: restore");
  cpgtext(uxmin+.65*dx, uymax+.07*dy, "space: print cursor position");
  cpgtext(uxmin+.65*dx, uymax+.04*dy, "s:     cycle smoothing mode");
  cpgtext(uxmin+.65*dx, uymax+.01*dy, "+:     smooth    -: unsmooth");

  cpgtext(uxmin+.40*dx, uymax+.04*dy, "./,: slide up/down");
  cpgtext(uxmin+.40*dx, uymax+.07*dy, "</>: zoom out/in");

  if (smooth > 1)
    {
      if (smooth_mode == S_AVERAGE)
	sprintf(smfac, "average %d", smooth);
      if (smooth_mode == S_MEDIAN)
	sprintf(smfac, "median %d", smooth);
      if (smooth_mode == S_REBIN)
	sprintf(smfac, "rebin %d", smooth);
      cpgtext(uxmin+.45*dx, uymax+.01*dy, smfac);
    }
      
  return;
}

static float s_mod (float a, float b)
{
  float mod;
  if (b != 0) 
    mod = a-(int)(a/b)*b;
  else 
    return (-1);
  return (mod);
}

static int s_comp_func (const void *a, const void *b)
{
  float *x, *y;
  x = (float *) a;
  y = (float *) b;

  if (*x < *y)
      return -1;
  else if (*x > *y)
      return  1;
  else
      return 0;
}

float s_median(float *list, int num)

{
  float median;
  static float mm = 2.0;

  if (num == 0 || list == NULL)
    {
      g_warning ("s_median(): Invalid input.");
      return (0.0);
    }

  qsort(list, num, sizeof(float), s_comp_func);

  /* Calculate median 
   */
  if (s_mod((float)num, mm) != 0)
    median = list[num/2];
  else
    median = (list[num/2 - 1] + list[num/2]) / 2.0;

  return(median);
}


/* smooth orig into xx, both having size nn, rebin factor 'factor'.
 */
static void l_smooth (float * xx, float * orig, int nn, int factor)
{
  float mean = 0.0;
  int   i;

  float med_list[64];
  
  int   quot;
  int   start;
  int   rem;
  int   j;
  

  if (factor == 1)
    {
      memcpy(xx, orig, nn * sizeof(float));
      return;
    }

  /* -- fast and dirty --
   */
  if (smooth_mode == S_AVERAGE)
    {
      /* -- running average -- */
      mean = 0;
      
      for (i = 0; i < factor; ++i)
	mean += orig[i];
      
      xx[0] = mean / factor;
      
      
      for (i = 1; i < (count - factor); ++i)
	{
	  mean -= orig[i-1];
	  mean += orig[i-1+factor];
	  xx[i] = mean / factor;
	}
      
      for (i = (count - factor); i < count; ++i)
	xx[i] = mean / factor;
      
      /* -- end running average -- */
    }

  else if (smooth_mode == S_MEDIAN)
    {
      if (factor > 64)
	{
	  g_warning("Median limited to 64 max.");
	  factor = 64;
	}

      memcpy (med_list, orig, factor * sizeof(float));
      mean  = s_median (med_list, factor);
      for (i = 0; i < (factor/2 + 1); ++i)
	xx[i] = mean;
      
      for (i = (factor/2 + 1); i < (nn-factor); ++i)
	{
	  memcpy (med_list, &orig[i - (factor/2)], factor * sizeof(float));
	  /* -- median -- */

	  mean  = s_median (med_list, factor);

	  xx[i] = mean;
	}
      
      for (i = (nn-factor); i < nn; ++i)
	xx[i] = mean;
    }

  else if (smooth_mode == S_REBIN)
    {
      quot = nn / factor;
      rem  = nn - (quot * factor);
      
      for (i = 0; i < quot; ++i)
	{
	  mean  = 0;
	  start = i * factor;
	  
	  for (j = 0; j < factor; ++j)
	    mean += orig[start+j];
	  
	  mean /= factor;
	  
	  for (j = 0; j < factor; ++j)
	    xx[start+j] = mean;
	}
      
      start = quot * factor;
      mean  = 0;
      
      for (j = 0; j < rem; ++j)
	mean += orig[start+j];
      
      mean /= factor;
      
      for (j = 0; j < rem; ++j)
	xx[start+j] = mean;
    }

  return;
}


  
/* ----------------------------------------------- */

int getlambda(float * xposition, float * yposition, 
	      int * numlines)
{
  int lines       = 0;

  int i;

  /*
  FILE * fd;
  char line[BUFSIZ];
  int items = 0;
  */

  float   xmax = 1e-8;
  float   xmin = 1e+8;
  float   ymax = 1e-8;
  float   ymin = 1e+8;

  float   uxmax;
  float   uxmin;
  float   uymax;
  float   uymin;


  float   * l_wave;
  float   * l_flux;
  int     l_count;

  int     haveit, index;
  float   uxpos, uypos;
  char    c;
  
  *yposition=0;

  index = 0;
  ymax = float_max(flux,count,&index); ymin = float_min(flux,count,&index);
  xmax = float_max(wave,count,&index); xmin = float_min(wave,count,&index);

  l_wave  = (float *) malloc (sizeof(float) * count);
  l_flux  = (float *) malloc (sizeof(float) * count);
  l_count = count;

  if (l_wave == NULL || l_flux == NULL)
    {
      g_warning("Out of memory");
      return (-1);
    }

  memcpy (l_wave, wave, count * sizeof(float));
  memcpy (l_flux, flux, count * sizeof(float));

  if (cpgopen("/XSERVE") != 1) 
    {
      g_warning("Cannot open plot device");
      free(l_wave); free(l_flux);
      return (-1);
    }

  uxmax = xmax;
  uymax = ymax;
  uxmin = xmin;
  uymin = ymin;

  uxpos = (uxmax+uxmin)/2;
  uypos = (uymax+uymin)/2;

  redraw_it (xposition, *yposition, lines,
	     l_wave, l_flux, l_count,
	     uxmin, uxmax, uymin, uymax);

  haveit = 0;

  while (1)
    {

      cpgband (7, 1, (uxmax+uxmin)/2, (uymax+uymin)/2, &uxpos, &uypos, &c);

      if (c == 'q' || c == 'Q')
	break;

      if (c == 'c')
        {
	  zoom  = 1.0;

          uxmax = xmax;
          uymax = ymax;
          uxmin = xmin;
          uymin = ymin;

          uxpos = (uxmax+uxmin)/2;
          uypos = (uymax+uymin)/2;

          haveit = 0;

	  redraw_it (xposition, *yposition, lines,
		     l_wave, l_flux, l_count,
		     uxmin, uxmax, uymin, uymax);
          continue;
        }

      /* zoom in (>) or out (<)
       */
      if (c == '>' || c == '<')
	{
	  haveit = 0;
	  
	  if (c == '>')
	    zoom *= 2.0;
	  else 
	    zoom /= 2.0;

	  zoom = (zoom < 1.0) ? 1.0 : zoom;

	  uymax = (uxmax + uxmin)/2;
	  uymin = (xmax - xmin)  / zoom;
	  uymin = uymin / 2.0;
	  uymin = (uymin < 0.0) ? (-uymin) : uymin; 

	  uxmax = uymax + uymin;
	  uxmin = uymax - uymin;

	  uymax = 1e-8;
	  uymin = 1e+8;

	  for (i = 0; i < count; ++i)
	    {
	      if (l_wave[i] >= uxmin && l_wave[i] <= uxmax)
		{
		  uymin = MIN(uymin, flux[i]);
		  uymax = MAX(uymax, flux[i]);
		}
	    }
	  redraw_it (xposition, *yposition, lines,
		     l_wave, l_flux, l_count,
		     uxmin, uxmax, uymin, uymax);
	  continue;
	}
      
      /* slide down (,) or up (.)
       */
      if (c == ',' || c == '.')
	{
	  haveit = 0;
	  
	  uymax = (uxmax - uxmin);
	  
	  if (c == '.')
	      uxmax += uymax * 0.8;
	  else
	      uxmax -= uymax * 0.8;

	  uxmax = MIN(xmax,         uxmax);
	  uxmax = MAX((xmin+uymax), uxmax);

	  uxmin = uxmax - uymax;
	  
	  uymax = 1e-8;
	  uymin = 1e+8;

	  for (i = 0; i < count; ++i)
	    {
	      if (l_wave[i] >= uxmin && l_wave[i] <= uxmax)
		{
		  uymin = MIN(uymin, flux[i]);
		  uymax = MAX(uymax, flux[i]);
		}
	    }
	  redraw_it (xposition, *yposition, lines,
		     l_wave, l_flux, l_count,
		     uxmin, uxmax, uymin, uymax);
	  continue;
	}
      
      /* --- cycle the smoothing mode ---
       */
      if (c == 's')
	{
	  ++smooth_mode;
	  if (smooth_mode > S_REBIN)
	    smooth_mode = 0;
	  l_smooth(l_wave, wave, count, smooth);
	  l_smooth(l_flux, flux, count, smooth);
	  redraw_it (xposition, *yposition, lines,
		     l_wave, l_flux, l_count,
		     uxmin, uxmax, uymin, uymax);
	  continue;
	}

      /* --- smooth ---
       */
      if (c == '+')
	{
	  if (smooth < count/2)
	    {
	      ++smooth;
	      l_smooth(l_wave, wave, count, smooth);
	      l_smooth(l_flux, flux, count, smooth);
	      redraw_it (xposition, *yposition, lines,
			 l_wave, l_flux, l_count,
			 uxmin, uxmax, uymin, uymax);
	    }
	  continue;
	}

      /* --- unsmooth ---
       */
      if (c == '-')
	{
	  if (smooth > 1)
	    {
	      --smooth;
	      l_smooth(l_wave, wave, count, smooth);
	      l_smooth(l_flux, flux, count, smooth);
	      redraw_it (xposition, *yposition, lines,
			 l_wave, l_flux, l_count,
			 uxmin, uxmax, uymin, uymax);
	    }
	  continue;
	}

      if (c == ' ')
        {
	  g_message("X = %f, Y = %f\n", uxpos, uypos);
	  continue;
         }

      if (c == MOUSE_M || c == 'b')
	{
	  *yposition = uypos;
	  g_message("Background = %f", *yposition);
	  redraw_it (xposition, *yposition, lines,
		     l_wave, l_flux, l_count,
		     uxmin, uxmax, uymin, uymax);
	  continue;
	}
      if (c == 'd')
	{
	  g_message("%02d  Lambda = %f deleted", 
		  lines, xposition[lines-1]);
	  if (lines > 0) --lines;
	  redraw_it (xposition, *yposition, lines,
		     l_wave, l_flux, l_count,
		     uxmin, uxmax, uymin, uymax);
	  continue;
	  
	}
      if (c == MOUSE_R || c == 'm')
	{
	  xposition[lines] = uxpos;
	  g_message("%02d  Lambda = %f\n", lines+1, xposition[lines]);
	  ++lines;
	  if (lines == MAXLINES)
	    break;
	  cpgsci(3);
	  for (i = 0; i < lines; ++i)
	    {
	      cpgmove(xposition[i], uymin);
	      cpgdraw(xposition[i], uymax);
	    }
	  cpgsci(1);
	  continue;
	}

      else if (c == MOUSE_L || c == 'a') 
	++haveit;
      else
	continue;
      
      if (haveit == 1 && (c == MOUSE_L || c == 'a'))
	{
	  uxmin = MAX(xmin, uxpos);
	}
      
      if (haveit == 2 && (c == MOUSE_L || c == 'a'))
	{
	  haveit = 0;
	  
	  uxmax = MIN(xmax, uxpos);
	  
	  if (uxmax < uxmin)
	    {
	      uymin = uxmin; uxmin = uxmax; uxmax = uymin;
	    }

	  zoom = (xmax - xmin) / (uxmax - uxmin);

	  uymax = 1e-8;
	  uymin = 1e+8;

	  for (i = 0; i < count; ++i)
	    {
	      if (l_wave[i] >= uxmin && l_wave[i] <= uxmax)
		{
		  uymin = MIN(uymin, flux[i]);
		  uymax = MAX(uymax, flux[i]);
		}
	    }
	  redraw_it (xposition, *yposition, lines,
		     l_wave, l_flux, l_count,
		     uxmin, uxmax, uymin, uymax);
	}
      
    }
  
  cpgend();
  
  *numlines = lines; 
  
  if (*numlines < 1) 
    g_warning ("%d Lines selected\n", *numlines);
  
  free(l_wave); free(l_flux); 
  return(0);
}



