#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include "cora_fit.h"

#ifndef M_1_PI
#define M_1_PI          0.31830988618379067154  /* 1/pi */
#endif

int replot(char *path, char *plotfile)
{

  int i, j, k, lline;
  FILE *fd;
  float f1, f2, f3, f4, f5, f6, f7;
  char entry_text[3 * MAXTEXTL];
  Vector y, gexp;
  double bweight, myy, myy2, myx;
  
  vec_init(&y);
  vec_init(&gexp);
  
  if (verbose)
    fprintf(stdout, "Message      : Reading spectrum from %s...\n", datafile);
  if (0 >= read_spec (datafile, 0))
    {
      fprintf(stderr,
              "* Warning *: No spectrum data read in.\n");
      return(-1);
    }
  
  
  if (verbose)
    fprintf(stdout, 
	    "Message      : Reading parameters from %s...\n", conffile);
  
  if (0 >= read_input (conffile, 0))
    {
      fprintf(stderr,
              "* Warning *: No parameter file read in.\n");
      return (-1);
    }
  
  binsize=(wave.vec[wave.len-1]-wave.vec[0])/(wave.len-1);
  fd = fopen (path, "r");
  i = errno;
  if (!fd)
    {
      printf("* Warning *: fopen: %s: %s\n",path,strerror (i));
      printf("*              will ignore option -r\n");
      vec_free(&y);
      vec_free(&gexp);
      vec_init(&llambda);
      vec_init(&erlam);
      vec_init(&lsigma);
      vec_init(&ersig);
      vec_init(&ampout);
      vec_init(&eramp);
      vec_init(&steplam);
      vec_init(&stepsig);
      vec_init(&wave);
      vec_init(&sp);
      vec_init(&err);
      vec_init(&bkgd);
      vec_init(&bkerr);
      vec_init(&bg0);
      return(-1);
    }
  i = 0;
  while (NULL != fgets (entry_text, 128, fd))
    {
      if (entry_text[0] == '#') continue;
      if (entry_text[0] == '\n') continue;
      if (entry_text[0] == '\t') continue;
      ++i;
    }
  fclose  (fd);
  
  fd = fopen (path, "r");
  nlines = i;
  
  vec_init(&llambda);
  vec_init(&erlam);
  vec_init(&lsigma);
  vec_init(&ersig);
  vec_init(&ampout);
  vec_init(&eramp);
  vec_alloc (&llambda, nlines);
  vec_alloc (&erlam, nlines);
  vec_alloc (&lsigma, nlines);
  vec_alloc (&ersig, nlines);
  vec_alloc (&ampout, nlines);
  vec_alloc (&eramp, nlines);
  
  i = 0;
  while (NULL != fgets (entry_text, 128, fd))
    {
      
      if (entry_text[0] == '#') continue;
      if (entry_text[0] == '\n') continue;
      if (entry_text[0] == '\t') continue;
      j = 0;
      while (entry_text[j] != '\0')
	{
          if (entry_text[j] == ',')
            entry_text[j] = ' ';
          ++j;
	}
      j = sscanf (entry_text, 
		  "%f %f %f %f %f %f %f %d", 
		  &f1, &f2, &f3, &f4, &f5, &f6, &f7, &k);
      if (j < 1)
	{
          fprintf (stderr, 
                   "* Warning *: Format error: %s, line %d: %s.\n", 
                   path, i - 1, entry_text);
          return(-1);
	}

      llambda.vec[i] = f1;
      erlam.vec[i]   = f2;
      lsigma.vec[i]  = f3;
      ersig.vec[i]   = f4;
      ampout.vec[i]  = f5;
      eramp.vec[i]   = f6;
      pro_type[i]    = k;
      if (sbgit > 0) 
	sbg=f7;
      ++i;
  }
  fclose(fd);

  lamran();
  vec_alloc(&y,count);
  vec_alloc(&gexp,count);

  /*  convert to units cts/bin  */
  if (sbgit>0) 
    sbg=sbg*(wave.vec[wave.len-1] - wave.vec[0])/wave.len;

  bweight = app_s/app_b;
  for (i=0; i<count; ++i)
    {
      bg0.vec[i] = bg0.vec[i] * bweight;
      bkgd.vec[i] = bkgd.vec[i] * bweight;
    }
  /*   define current environment  */
  for (i=0;i<count;++i)
    {
      x.vec[i]      = wave.vec[(int)good.vec[i]];
      ncount.vec[i] =   sp.vec[(int)good.vec[i]];
      bbb.vec[i]    = bkgd.vec[(int)good.vec[i]];
      bb0.vec[i]    =  bg0.vec[(int)good.vec[i]];
    }
  if (bgsmooth>0) 
    smoothbg();

  /*  add source background to data interval in consideration */
  for (i=0; i < count; ++i)
     bbb.vec[i] = bbb.vec[i]+sbg;

  for (lline=0; lline < nlines; ++lline)
    {
      if (pro_type[lline] == P_GAUSS)
	{
	  for(i=0;i<count;++i)
	    gexp.vec[i] = -(x.vec[i] - llambda.vec[lline])/lsigma.vec[lline];
	  gaussint(&gexp,&y);
      
	  yc[lline][0]       = 0.;
	  yc[lline][count-1] = 0.;
	  for (i=1; i < (count-1); ++i)
	      yc[lline][i]  = (y.vec[i-1] - y.vec[i+1]) / 2.0;
	}
      else if (pro_type[lline] == P_LORENZ)
	{
	  myy  = lsigma.vec[lline] / 2.0;
	  myy2 = myy * myy;
	  
	  y.vec[0] = 0.0;
	  
	  for(i=1; i < count; ++i)
	    {
	      myx = (x.vec[i] - llambda.vec[lline]);

	      /* Lorentz profile
	       */
	      yc[lline][i] = (M_1_PI * myy) / ( (myx * myx) + myy2);
	      
	      /* Take binsize into account
	       */
	      yc[lline][i] *= (x.vec[i] - x.vec[i-1]);
		
	    }
	}

      if (sigunit == 1) 
	lsigma.vec[lline] = lsigma.vec[lline]*C_LIGHT/llambda.vec[lline];
    }
      
/*   print out yc
for(i=0;i<nlines;++i)
 {for(j=0;j<count;++j) printf("%f  ",yc[i][j]);printf("\n");} */


  icount = 1;
  
  plotfit (conffile       /* inputfile                        */,
           plotfile	  /* filename if other than window    */,
           iformat        /* XSERVE/PS/GIF/PPM                */
           );

  return(0);
}

